@@ -20,12 +20,14 @@ import urllib2
 import zipfile
 import shutil
 import tempfile
+import errno
 from urlparse import urlparse, urljoin
 
 from esky.bootstrap import parse_version, join_app_version
 from esky.errors import *
 from esky.util import deep_extract_zipfile, copy_ownership_info, \
-                      ESKY_CONTROL_DIR
+                      ESKY_CONTROL_DIR, ESKY_APPDATA_DIR, \
+                      really_rmtree, really_rename
 from esky.patch import apply_patch, PatchError
 
 
@@ -40,6 +42,9 @@ class VersionFinder(object):
         fetch_version:  make the specified version available locally
                         (e.g. download it from the internet)
 
+        fetch_version_iter:  like fetch_version but yielding progress updates
+                             during its execution
+
         has_version:  check that the specified version is available locally
 
         cleanup:  perform maintenance/cleanup tasks in the workdir
@@ -89,7 +94,7 @@ class VersionFinder(object):
     def has_version(self,app,version):
         """Check whether a specific version of the app is available locally.
 
-        Returns either False, or the paths to the unpacked version directory.
+        Returns either False, or the path to the unpacked version directory.
         """
         raise NotImplementedError
 
@@ -103,10 +108,8 @@ class DefaultVersionFinder(VersionFinder):
     downloaded and extracted on request.
 
     Zipfiles suitable for use with this class can be produced using the
-    "bdist_esky" distutils command.
-
-    This class will eventually grow support for applying differential updates,
-    but I haven't implemented it yet...
+    "bdist_esky" distutils command.  It also supports simple differential
+    updates as produced by the "bdist_esky_patch" command.
     """
 
     def __init__(self,download_url):
@@ -123,7 +126,7 @@ class DefaultVersionFinder(VersionFinder):
                 try:
                     os.mkdir(target)
                 except OSError, e:
-                    if e.errno not in (17,183):
+                    if e.errno not in (errno.EEXIST,183):
                         raise
                 else:
                     copy_ownership_info(app.appdir,target)
@@ -152,10 +155,10 @@ class DefaultVersionFinder(VersionFinder):
             os.unlink(os.path.join(dldir,nm))
         updir = self._workdir(app,"unpack")
         for nm in os.listdir(updir):
-            shutil.rmtree(os.path.join(updir,nm))
+            really_rmtree(os.path.join(updir,nm))
         rddir = self._workdir(app,"ready")
         for nm in os.listdir(rddir):
-            shutil.rmtree(os.path.join(rddir,nm))
+            really_rmtree(os.path.join(rddir,nm))
 
     def open_url(self,url):
         f = urllib2.urlopen(url)
@@ -208,7 +211,7 @@ class DefaultVersionFinder(VersionFinder):
                         yield status
             try:
                 self._prepare_version(app,version,local_path)
-            except PatchError:
+            except (PatchError,EskyVersionError,EnvironmentError):
                 yield {"status":"retrying","size":None}
         yield {"status":"ready","path":name}
 
@@ -217,8 +220,15 @@ class DefaultVersionFinder(VersionFinder):
         outfilenm = os.path.join(self._workdir(app,"downloads"),nm)
         if not os.path.exists(outfilenm):
             infile = self.open_url(urljoin(self.download_url,url))
-            if not hasattr(infile,"size"):
-                infile.size = None
+            try:
+                infile_size = infile.size
+            except AttributeError:
+                try:
+                    fh = infile.fileno()
+                except AttributeError:
+                    infile_size = None
+                else:
+                    infile_size = os.fstat(fh).st_size
             try:
                 partfilenm = outfilenm + ".part"
                 partfile = open(partfilenm,"wb")
@@ -226,7 +236,7 @@ class DefaultVersionFinder(VersionFinder):
                     data = infile.read(1024*512)
                     while data:
                         yield {"status": "downloading",
-                               "size": infile.size,
+                               "size": infile_size,
                                "received": partfile.tell(),
                         }
                         partfile.write(data)
@@ -237,7 +247,7 @@ class DefaultVersionFinder(VersionFinder):
                     raise
                 else:
                     partfile.close()
-                    os.rename(partfilenm,outfilenm)
+                    really_rename(partfilenm,outfilenm)
             finally:
                 infile.close()
         yield {"status":"ready","path":outfilenm}
@@ -252,6 +262,7 @@ class DefaultVersionFinder(VersionFinder):
         uppath = tempfile.mkdtemp(dir=self._workdir(app,"unpack"))
         try:
             if not path:
+                #  There's nothing to prepare, just copy the current version.
                 self._copy_best_version(app,uppath)
             else:
                 if path[0][0].endswith(".patch"):
@@ -277,46 +288,103 @@ class DefaultVersionFinder(VersionFinder):
                             pass
                         raise
                     patches = path[1:]
-                for (patchfile,patchurl) in patches:
-                    try:
-                        with open(patchfile,"rb") as f:
-                            apply_patch(uppath,f)
-                    except PatchError:
-                        self.version_graph.remove_all_links(patchurl)
+                # TODO: remove compatability hooks for ESKY_APPDATA_DIR="".
+                # If a patch fails to apply because we've put an appdata dir
+                # where it doesn't expect one, try again with old layout. 
+                for _ in xrange(2):
+                    #  Apply any patches in turn.
+                    for (patchfile,patchurl) in patches:
                         try:
-                            os.unlink(patchfile)
-                        except EnvironmentError:
-                            pass
-                        raise
-            # Move anything that's not the version dir into esky/bootstrap
+                            try:
+                                with open(patchfile,"rb") as f:
+                                    apply_patch(uppath,f)
+                            except EnvironmentError, e:
+                                if e.errno not in (errno.ENOENT,):
+                                    raise
+                                if not path[0][0].endswith(".patch"):
+                                    raise
+                                really_rmtree(uppath)
+                                os.mkdir(uppath)
+                                self._copy_best_version(app,uppath,False)
+                                break
+                        except (PatchError,EnvironmentError):
+                            self.version_graph.remove_all_links(patchurl)
+                            try:
+                                os.unlink(patchfile)
+                            except EnvironmentError:
+                                pass
+                            raise
+                    else:
+                        break
+            # Find the actual version dir that we're unpacking.
+            # TODO: remove compatability hooks for ESKY_APPDATA_DIR=""
             vdir = join_app_version(app.name,version,app.platform)
-            bspath = os.path.join(uppath,vdir,ESKY_CONTROL_DIR,"bootstrap")
+            vdirpath = os.path.join(uppath,ESKY_APPDATA_DIR,vdir)
+            if not os.path.isdir(vdirpath):
+                vdirpath = os.path.join(uppath,vdir)
+                if not os.path.isdir(vdirpath):
+                    self.version_graph.remove_all_links(path[0][1])
+                    err = version + ": version directory does not exist"
+                    raise EskyVersionError(err)
+            # Move anything that's not the version dir into "bootstrap" dir.
+            ctrlpath = os.path.join(vdirpath,ESKY_CONTROL_DIR)
+            bspath = os.path.join(ctrlpath,"bootstrap")
             if not os.path.isdir(bspath):
                 os.makedirs(bspath)
             for nm in os.listdir(uppath):
-                if nm != vdir:
-                    os.rename(os.path.join(uppath,nm),os.path.join(bspath,nm))
+                if nm != vdir and nm != ESKY_APPDATA_DIR:
+                    really_rename(os.path.join(uppath,nm),
+                                  os.path.join(bspath,nm))
             # Check that it has an esky-files/bootstrap-manifest.txt file
-            bsfile = os.path.join(uppath,vdir,ESKY_CONTROL_DIR,"bootstrap-manifest.txt")
+            bsfile = os.path.join(ctrlpath,"bootstrap-manifest.txt")
             if not os.path.exists(bsfile):
                 self.version_graph.remove_all_links(path[0][1])
-                err = "patch didn't create bootstrap-manifest.txt"
-                raise PatchError(err)
-            # Make it available for upgrading
+                err = version + ": version has no bootstrap-manifest.txt"
+                raise EskyVersionError(err)
+            # Make it available for upgrading, replacing anything
+            # that we previously had available.
             rdpath = self._ready_name(app,version)
-            if os.path.exists(rdpath):
-                shutil.rmtree(rdpath)
-            os.rename(os.path.join(uppath,vdir),rdpath)
+            tmpnm = None
+            try:
+                if os.path.exists(rdpath):
+                    tmpnm = rdpath + ".old"
+                    while os.path.exists(tmpnm):
+                        tmpnm = tmpnm + ".old"
+                    really_rename(rdpath,tmpnm)
+                really_rename(vdirpath,rdpath)
+            finally:
+                if tmpnm is not None:
+                    really_rmtree(tmpnm)
+            #  Clean up any downloaded files now that we've used them.
             for (filenm,_) in path:
                 os.unlink(filenm)
         finally:
-            shutil.rmtree(uppath)
+            really_rmtree(uppath)
 
-    def _copy_best_version(self,app,uppath):
+    def _copy_best_version(self,app,uppath,force_appdata_dir=True):
+        """Copy the best version directory from the given app.
+
+        This copies the best version directory from the given app into the
+        unpacking path.  It's useful for applying patches against an existing
+        version.
+        """
         best_vdir = join_app_version(app.name,app.version,app.platform)
-        source = os.path.join(app.appdir,best_vdir)
-        shutil.copytree(source,os.path.join(uppath,best_vdir))
-        with open(os.path.join(source,ESKY_CONTROL_DIR,"bootstrap-manifest.txt"),"r") as manifest:
+        #  TODO: remove compatability hooks for ESKY_APPDATA_DIR="".
+        source = os.path.join(app.appdir,ESKY_APPDATA_DIR,best_vdir)
+        if not os.path.exists(source):
+            source = os.path.join(app.appdir,best_vdir)
+        if not force_appdata_dir:
+            dest = uppath
+        else:
+            dest = os.path.join(uppath,ESKY_APPDATA_DIR)
+        try:
+            os.mkdir(dest)
+        except OSError, e:
+            if e.errno not in (errno.EEXIST,183):
+                raise
+        shutil.copytree(source,os.path.join(dest,best_vdir))
+        mfstnm = os.path.join(source,ESKY_CONTROL_DIR,"bootstrap-manifest.txt")
+        with open(mfstnm,"r") as manifest:
             for nm in manifest:
                 nm = nm.strip()
                 bspath = os.path.join(app.appdir,nm)
@@ -364,27 +432,8 @@ class LocalVersionFinder(DefaultVersionFinder):
                 self.version_graph.add_link(from_version or "",version,nm,cost)
         return self.version_graph.get_versions(app.version)
 
-    def _fetch_file(self,app,nm):
-        infile = open(os.path.join(self.download_url,nm),"rb")
-        outfilenm = os.path.join(self._workdir(app,"downloads"),nm)
-        if not os.path.exists(outfilenm):
-            partfilenm = outfilenm + ".part"
-            partfile = open(partfilenm,"wb")
-            try:
-                data = infile.read(1024*512)
-                while data:
-                    partfile.write(data)
-                    data = infile.read(1024*512)
-            except Exception:
-                infile.close()
-                partfile.close()
-                os.unlink(partfilenm)
-                raise
-            else:
-                infile.close()
-                partfile.close()
-                os.rename(partfilenm,outfilenm)
-        return outfilenm
+    def open_url(self,url):
+        return open(os.path.join(self.download_url,url),"rb")
 
 
 class VersionGraph(object):
